{% extends "./layout.html" %}

{% comment %}
Copyright (C) 2017-2020  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% load swh_templatetags %}

{% block title %}{{ heading }} &ndash; Software Heritage archive{% endblock %}

{% block navbar-content %}

<h4>
  Browse the archive
</h4>

{% endblock %}

{% block browse-content %}

{% block swh-browse-before-content %}
{% if snapshot_context %}
  {% include "includes/snapshot-context.html" %}
{% endif %}
{% endblock %}

{% block swh-browse-content %}{% endblock %}

{% block swh-browse-after-content %}{% endblock %}

<script>
  swh.webapp.initPage('browse');
</script>

{% endblock %}
