{% extends "./layout.html" %}

{% comment %}
Copyright (C) 2017-2019  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% load render_bundle from webpack_loader %}

{% block navbar-content %}
<h4>Download archived software</h4>
{% endblock %}

{% block browse-content %}
<p>
  This interface enables to track the status of the different Software Heritage
  Vault cooking tasks created while browsing the archive.
</p>
<p>
  Once a cooking task is finished, a link will be made available in order to
  download the associated archive.
</p>
<button type="button" class="btn btn-default btn-sm" id="vault-remove-tasks">Remove selected tasks</button>
<div class="table-responsive mt-3">
  <table class="table swh-table swh-table-striped swh-vault-table" id="vault-cooking-tasks">
    <thead>
      <tr>
        <th>
          <div class="custom-control custom-checkbox">
            <input type="checkbox" class="custom-control-input" id="vault-tasks-toggle-selection">
            <label class="custom-control-label" for="vault-tasks-toggle-selection"></label>
          </div>
        </th>
        <th style="width: 300px">Origin</th>
        <th style="width: 100px">Object type</th>
        <th>Object info</th>
        <th style="width: 250px">Cooking status</th>
        <th style="width: 120px"></th>
      </tr>
    </thead>
    <tbody></tbody>
  </table>
</div>
{% include "includes/vault-common.html" %}
<script>
  swh.webapp.initPage('vault');
  swh.vault.initUi();
</script>
{% endblock %}
