# Generated by Django 2.2.19 on 2021-03-22 15:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("swh_web_auth", "0002_remove_stored_tokens"),
    ]

    operations = [
        migrations.DeleteModel(name="OIDCUser",),
    ]
