# Copyright (C) 2021 The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information
# Generated by Django 2.2.19 on 2021-04-19 16:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("swh_web_common", "0008_save-code-now_indexes_20210106_1327"),
    ]

    operations = [
        migrations.AddField(
            model_name="saveoriginrequest",
            name="visit_status",
            field=models.TextField(
                choices=[
                    ("created", "created"),
                    ("ongoing", "ongoing"),
                    ("full", "full"),
                    ("partial", "partial"),
                    ("not_found", "not_found"),
                    ("failed", "failed"),
                ],
                null=True,
            ),
        ),
    ]
